@@ -17,40 +17,62 @@
 * 02111-1307, USA.
 */
 
-kukit.sa = {};
+kukit.sa = new function() {   /// MODULE START
 
-kukit.sa.ServerAction = function(name, oper) {
+var sa = this;
+
+// ServerActionBase is a non-initialize version of ServerAction
+// this enables to subclass it or reuse its methods
+sa.ServerActionBase = function() {
+
+this.initialize = function(name, oper) {
+    this.oper = oper;
     this.url = oper.kssParms.kssUrl;
     if (typeof(this.url) == 'undefined') {
         this.url = name;
     }
     this.url = this.calculateAbsoluteURL(this.url);
-    this.oper = oper;
     this.notifyServer();
 };
 
-kukit.sa.ServerAction.prototype.calculateAbsoluteURL = function(url) {
-    //
-    // If the url is an absolute path, it is used
-    //
-    // If the url is not an absolute path, it is put at the end of the context
-    // url.
-    //
-    // example: url='@theview/getName',
-    //          context='http://your.site.com/portal/folder/object'
-    //
-    //     result='http://your.site.com/portal/folder/object/@@theview/getName'
-    //
-    if (url.match(RegExp('/^https?:\/\//'))) {
+/* 
+ * calculateAbsoluteURL
+ *
+ * Makes absolute site url
+ * - if starts with http:// https:// : no change
+ * - if starts with /: interprets absolute from domain
+ * - otherwise: relative to current context
+ */
+this.calculateAbsoluteURL = function(url) {
+    // XXX packer breaks on following regexp constant,
+    // so it must be quoted
+    if (url.match(RegExp("/^https?:\/\//"))) {
+        // absolute already
         return url;
-    } else {
-        var result = kukit.engine.baseUrl + '/' + url;
-        return result;
     }
+    var absoluteMatch = url.match(RegExp(/^(\/)(.*)/));
+    var path = kukit.engine.baseUrl;
+    if (absoluteMatch) {
+        // relative to domain
+        var base = path.match(RegExp(/^(.*:\/\/[^\/]*)(\/?)/))[1];
+        // base is like: http://foo.bar without trailing /
+        url = base + url;
+        return url;
+    }
+    // final case: relative to current url
+    // (paranoia: add an / to the path *only* if it is
+    // *not* there) 
+    // XXX packer breaks on following
+    // regexp constant, so it must be quoted
+    if (! path.match(RegExp("/$"))) {
+        path += '/';
+    }
+    url = path + url;
+    return url;
 };
 
 // Backparameters can be used on command execution.
-kukit.sa.ServerAction.prototype.notifyServer = function() {
+this.notifyServer = function() {
     var self = this;
     var sendHook = function(queueItem) {
         // store the queue reception on the oper
@@ -65,7 +87,7 @@ kukit.sa.ServerAction.prototype.notifyServer = function() {
     kukit.engine.requestManager.notifyServer(sendHook, this.url, timeoutHook);
 };
 
-kukit.sa.ServerAction.prototype.reallyNotifyServer = function() {
+this.reallyNotifyServer = function() {
     // make a deferred callback
     var domDoc = new XMLHttpRequest();
     var self = this;
@@ -100,9 +122,9 @@ kukit.sa.ServerAction.prototype.reallyNotifyServer = function() {
     domDoc.send(encoded);
 };
 
-kukit.sa.ServerAction.prototype.notifyServer_done = function(domDoc) {
-;;; var msg = 'Request readyState = ' + domDoc.readyState + '.';
-;;; kukit.logDebug(msg);
+this.notifyServer_done = function(domDoc) {
+//;;; var msg = 'Request readyState = ' + domDoc.readyState + '.';
+//;;; kukit.logDebug(msg);
     if (domDoc.readyState == 4) {
         // notify the queue that we are done
         var success = this.oper.queueItem.receivedResult();
@@ -111,33 +133,29 @@ kukit.sa.ServerAction.prototype.notifyServer_done = function(domDoc) {
         if (success) {
             // catch the errors otherwise won't get logged.
             // In FF they seem to get swallowed silently.
-;;;         try {
+            // We need these both in production and development mode,
+            // since the erorr fallbacks are activated from processError.
+            try {
                 // process the results
                 this.processResult(domDoc);
-;;;         } catch(e) {
+            } catch(e) {
 ;;;             if (e.name == 'RuleMergeError' || e.name == 'EventBindError') {
-;;;                 // Log the message
-;;;                 var msg = 'Error setting up events: ' + e.toString();
-;;;                 kukit.logFatal(msg);
-;;;                 // just throw it too...
-;;;                 throw msg;
-;;;             } else if (e.name == 'ResponseParsingError') {
-;;;                 this.processError('Response parsing error: ' + e);
-;;;             } else if (e.name == 'ExplicitError') {
-;;;                 this.processError(e.errorcommand);
-;;;             } else {
-;;;                 var msg = 'Unhandled error during command execution: ' + e;
-;;;                 kukit.logError(msg);
-;;;                 // also IE acts foul on thrown errors
-;;;                 // but at least mumbles something
-;;;                 throw e;
-;;;             }
-;;;         }
+;;;                 throw kukit.err.eventSetupError(e);
+;;;             } 
+                if (e.name == 'ResponseParsingError') {
+;;;                 kukit.E = 'Response parsing error: ' + e;
+                    this.processError(kukit.E);
+                } else if (e.name == 'ExplicitError') {
+                    this.processError(e.info.kw.errorcommand);
+                } else {
+                    throw e;
+                }
+            }
         }
-    }
+    };
 };
 
-kukit.sa.ServerAction.prototype.processResult = function(domDoc) {
+this.processResult = function(domDoc) {
     // checking various dom process errors, and get the commands part
     var dom;
     var commandstags = [];
@@ -159,9 +177,8 @@ kukit.sa.ServerAction.prototype.processResult = function(domDoc) {
             try {
                 dom = (new DOMParser()).parseFromString(payload, "text/xml");
             } catch(e) {
-;;;             var msg = 'Error parsing X-KSSCOMMANDS header.';
-;;;             kukit.E = new kukit.err.ResponseParsingError(msg);
-                throw kukit.E;
+;;;             kukit.E = 'Error parsing X-KSSCOMMANDS header.';
+                throw kukit.err.responseParsingError(kukit.E);
             }
             commandstags = kukit.dom.getNsTags(dom, 'commands');
             if (commandstags.length != 1) {
@@ -174,21 +191,20 @@ kukit.sa.ServerAction.prototype.processResult = function(domDoc) {
             // and log it as reported from the dom
             // Opera <= 8.5 does not have the parseError attribute,
             // so check for it first
-            dom = domDoc.responseXML;
-;;;         var msg = 'Unknown server error (invalid KSS response, no error';
-;;;         msg += ' info received)';
+;;;         dom = domDoc.responseXML;
+;;;         kukit.E = 'Unknown server error (invalid KSS response, no error';
+;;;         kukit.E += ' info received)';
 ;;;         if (dom && dom.parseError && (dom.parseError != 0)) {
-;;;             msg += ' : ' + Sarissa.getParseErrorText(dom);
+;;;             kukit.E += ' : ' + Sarissa.getParseErrorText(dom);
 ;;;             }
-;;;         kukit.E = new kukit.err.ResponseParsingError(msg);
-            throw kukit.E;
+            throw kukit.err.responseParsingError(kukit.E);
         }
-    }
+    };
+
     if (dom == null) {
         // this should not happen
-;;;     var msg = 'Neither xml nor html payload.';
-;;;     kukit.E = new kukit.err.ResponseParsingError(msg);
-        throw kukit.E;
+;;;     kukit.E = 'Neither xml nor html payload.';
+        throw kukit.err.responseParsingError(msg);
     }
     // find the commands (atm we don't limit ourselves inside the commandstag)
     var commands = kukit.dom.getNsTags(dom, 'command');
@@ -200,10 +216,12 @@ kukit.sa.ServerAction.prototype.processResult = function(domDoc) {
     // One or more valid commands to parse
     var command_processor = new kukit.cp.CommandProcessor();
     command_processor.parseCommands(commands, domDoc);
+    kukit.engine.beginSetupEventsCollection();
     command_processor.executeCommands(this.oper);
+    kukit.engine.finishSetupEventsCollection();
 };
 
-kukit.sa.ServerAction.prototype.processError = function(errorcommand) {
+this.processError = function(errorcommand) {
     var error_action = null;
     if (this.oper.eventRule) {
         var error_action = this.oper.eventRule.actions.getErrorActionFor(
@@ -215,6 +233,9 @@ kukit.sa.ServerAction.prototype.processError = function(errorcommand) {
 ;;;     reason = ', client_reason="' + errorcommand + '" ';
 ;;; } else if (typeof(errorcommand) != 'undefined') {
 ;;;     // a real error command, sent by the server
+;;;     // as kukit payload.
+;;;     // this way the server sends whatever message he wants as a parameter
+;;;     // to the error command.
 ;;;     reason = ', server_reason="' + errorcommand.parms.message + '" ';
 ;;; }
     if (error_action) {
@@ -229,6 +250,22 @@ kukit.sa.ServerAction.prototype.processError = function(errorcommand) {
 ;;;     kukit.E = 'Request failed at url ' + this.oper.queueItem.url;
 ;;;     kukit.E += ', rid=' + this.oper.queueItem.rid + reason;
 ;;;     kukit.logError(kukit.E);
+;;;     return;
+        // in case of no logging, we would like to throw an error.
+        // This means user will see something went wrong.
+        // XXX But: throwing an error on Firefox
+        // _seems to be ineffective__
+        // and throwing the error from IE
+        // _throws an ugly window, "Uncaught exception"
+        // TODO figure out something?
     }
 };
 
+};
+
+sa.ServerAction = function() {
+    this.initialize.apply(this, arguments);
+};
+sa.ServerAction.prototype = new sa.ServerActionBase();
+
+}();                              /// MODULE END
